
#include "stack.h"

/* Pointer to keep track of linked list front node.*/
static Node *pFront = NULL;

/* Function to add a new node to linked list at front.*/
static int addNodeToFront(Node **pFront, int iValue) {

    if(!pFront) {

        printf("\n Error : %s, %d, %s :: pFront = NULL\n", \
                __FILE__, __LINE__, __func__);
        return -1;

    }

    Node *pNewNode = malloc(sizeof(Node));
    
    if(!pNewNode) {

        printf("\n Error : %s, %d, %s :: pNewNode = NULL\n", \
                __FILE__, __LINE__, __func__);
        return -1;
    
    }

    pNewNode->iValue = iValue;
    
    if(!*pFront) {

        pNewNode->pNext  = NULL;
        *pFront          = pNewNode;

    }
    else {

        pNewNode->pNext = *pFront;
        *pFront         = pNewNode;

    }

    return 0; 

}

/* Function to delete a node from linked list front.*/
static int delNodeFromFront(Node **pFront, int *piValue) {

    if(!pFront) {

        printf("\n Error : %s, %d, %s :: pFront = NULL\n", \
                __FILE__, __LINE__, __func__);
        return -1;

    }

    if(!piValue) {

        printf("\n Error : %s, %d, %s :: piValue = NULL\n", \
                __FILE__, __LINE__, __func__);
        return -1;

    }

    if(!*pFront){

        printf(" Empty stack, nothing to pop.\n");
        return 1;
    
    }
    else {

        *piValue    = (*pFront)->iValue;
        Node *pNode = *pFront;
        *pFront     = pNode->pNext;
        free(pNode);
        
    }

    return 0;
 
}

/* Function to show stack content.*/
static int showStack() {

    if(!pFront)
        printf(" Empty stack, nothing to show.\n");   
    else {

        Node *pTempNode;
        unsigned int i;
        
        for(pTempNode = pFront, i = 0; pTempNode; \
            pTempNode = pTempNode->pNext, ++i)
                printf(" %u :  %d\n", (i+1), pTempNode->iValue);
 
    }

    return 0;

}

/* Function to push an integer to stack.*/
static int push(int iValue) {

    return addNodeToFront(&pFront, iValue);

}

/* Function to pop an integer from stack.*/
static int pop(int *piValue) {

    return delNodeFromFront(&pFront, piValue);

}

/* Function to print a choice menu.*/
static unsigned int cmdlnGUI() {

    printf(" ###################\n");
    printf(" 1. Push an integer,\n");
    printf(" 2. Pop an integer,\n");
    printf(" 3. Show the stack,\n");
    printf(" 4. Exit.\n");
    printf(" ###################\n");
    
    unsigned int uiChoice = 0;
    
    printf(" Enter a choice : ");
    scanf("%u", &uiChoice);

    return uiChoice; 

}

/* Function to test functionality of stack.*/
int test() {

    while(1) {

        unsigned int uiChoice = cmdlnGUI();
        
        if((uiChoice < 1) || (uiChoice > 4))
            printf("\n Invalid choice, enter again.\n");
        else {
            
            if(1 == uiChoice) {

                int iValue = 0;
                printf(" Enter the integer to push : ");
                scanf("%d", &iValue);

                if(-1 == push(iValue))
                    return -1;

            }
            else if(2 == uiChoice) {

                int iValue = 0;
                int iRet   = pop(&iValue);
                if(-1 == iRet)
                    return -1;

                if(0 == iRet)
                    printf(" The integer popped : %d\n", iValue);

            }
            else if(3 == uiChoice)
                showStack();
            
            else if(4 == uiChoice)
                return 0;

        }

    }

}

